@@ -46,13 +46,14 @@
 #include "nsWeakPtr.h"
 #include "nsVoidArray.h"
 #include "nsIDOMXMLDocument.h"
+#include "nsIDOM3Document.h"
 #include "nsIDOMDocumentView.h"
 #include "nsIDOMDocumentXBL.h"
 #include "nsIDOMNSDocument.h"
 #include "nsIDOMDocumentStyle.h"
 #include "nsIDOMDocumentRange.h"
 #include "nsIDOMDocumentTraversal.h"
-#include "nsIDocumentObserver.h"
+#include "nsStubDocumentObserver.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOM3EventTarget.h"
 #include "nsIDOMStyleSheetList.h"
@@ -79,6 +80,10 @@
 #include "nsIRadioGroupContainer.h"
 #include "nsIScriptEventManager.h"
 
+// Put these here so all document impls get them automatically
+#include "nsIHTMLStyleSheet.h"
+#include "nsIHTMLCSSStyleSheet.h"
+
 #include "pldhash.h"
 
 
@@ -96,6 +101,7 @@ class nsIDTD;
 class nsXPathDocumentTearoff;
 class nsIRadioVisitor;
 class nsIFormControl;
+class nsStyleSet;
 struct nsRadioGroupStruct;
 
 
@@ -138,7 +144,7 @@ protected:
 
 
 class nsDOMStyleSheetList : public nsIDOMStyleSheetList,
-                            public nsIDocumentObserver
+                            public nsStubDocumentObserver
 {
 public:
   nsDOMStyleSheetList(nsIDocument *aDocument);
@@ -148,62 +154,12 @@ public:
 
   NS_DECL_NSIDOMSTYLESHEETLIST
 
-  NS_IMETHOD BeginUpdate(nsIDocument *aDocument) { return NS_OK; }
-  NS_IMETHOD EndUpdate(nsIDocument *aDocument) { return NS_OK; }
-  NS_IMETHOD BeginLoad(nsIDocument *aDocument) { return NS_OK; }
-  NS_IMETHOD EndLoad(nsIDocument *aDocument) { return NS_OK; }
-  NS_IMETHOD BeginReflow(nsIDocument *aDocument,
-                         nsIPresShell* aShell) { return NS_OK; }
-  NS_IMETHOD EndReflow(nsIDocument *aDocument,
-                       nsIPresShell* aShell) { return NS_OK; }
-  NS_IMETHOD ContentChanged(nsIDocument *aDocument,
-                            nsIContent* aContent,
-                            nsISupports* aSubContent) { return NS_OK; }
-  NS_IMETHOD ContentStatesChanged(nsIDocument* aDocument,
-                                  nsIContent* aContent1,
-                                  nsIContent* aContent2,
-                                  PRInt32 aStateMask) { return NS_OK; }
-  NS_IMETHOD AttributeChanged(nsIDocument *aDocument,
-                              nsIContent*  aContent,
-                              PRInt32      aNameSpaceID,
-                              nsIAtom*     aAttribute,
-                              PRInt32      aModType,
-                              nsChangeHint aHint) { return NS_OK; }
-  NS_IMETHOD ContentAppended(nsIDocument *aDocument,
-                             nsIContent* aContainer,
-                             PRInt32     aNewIndexInContainer)
-                             { return NS_OK; }
-  NS_IMETHOD ContentInserted(nsIDocument *aDocument,
-                             nsIContent* aContainer,
-                             nsIContent* aChild,
-                             PRInt32 aIndexInContainer) { return NS_OK; }
-  NS_IMETHOD ContentReplaced(nsIDocument *aDocument,
-                             nsIContent* aContainer,
-                             nsIContent* aOldChild,
-                             nsIContent* aNewChild,
-                             PRInt32 aIndexInContainer) { return NS_OK; }
-  NS_IMETHOD ContentRemoved(nsIDocument *aDocument,
-                            nsIContent* aContainer,
-                            nsIContent* aChild,
-                            PRInt32 aIndexInContainer) { return NS_OK; }
-  NS_IMETHOD StyleSheetAdded(nsIDocument *aDocument,
-                             nsIStyleSheet* aStyleSheet);
-  NS_IMETHOD StyleSheetRemoved(nsIDocument *aDocument,
+  // nsIDocumentObserver
+  virtual void DocumentWillBeDestroyed(nsIDocument *aDocument);
+  virtual void StyleSheetAdded(nsIDocument *aDocument,
                                nsIStyleSheet* aStyleSheet);
-  NS_IMETHOD StyleSheetApplicableStateChanged(nsIDocument *aDocument,
-                                              nsIStyleSheet* aStyleSheet,
-                                              PRBool aApplicable) { return NS_OK; }
-  NS_IMETHOD StyleRuleChanged(nsIDocument *aDocument,
-                              nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aOldStyleRule,
-                              nsIStyleRule* aNewStyleRule) { return NS_OK; }
-  NS_IMETHOD StyleRuleAdded(nsIDocument *aDocument,
-                            nsIStyleSheet* aStyleSheet,
-                            nsIStyleRule* aStyleRule) { return NS_OK; }
-  NS_IMETHOD StyleRuleRemoved(nsIDocument *aDocument,
-                              nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aStyleRule) { return NS_OK; }
-  NS_IMETHOD DocumentWillBeDestroyed(nsIDocument *aDocument);
+  virtual void StyleSheetRemoved(nsIDocument *aDocument,
+                                 nsIStyleSheet* aStyleSheet);
 
 protected:
   PRInt32       mLength;
@@ -231,7 +187,7 @@ class nsDocument : public nsIDocument,
                    public nsIDOMDocumentRange,
                    public nsIDOMDocumentTraversal,
                    public nsIDOMDocumentXBL,
-                   public nsIDOM3Node,
+                   public nsIDOM3Document,
                    public nsSupportsWeakReference,
                    public nsIDOMEventReceiver,
                    public nsIDOM3EventTarget,
@@ -241,41 +197,28 @@ class nsDocument : public nsIDocument,
 public:
   NS_DECL_ISUPPORTS
 
-  NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
+  virtual void Reset(nsIChannel *aChannel, nsILoadGroup *aLoadGroup);
+  virtual void ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
 
-  NS_IMETHOD Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup);
-  NS_IMETHOD ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
+  virtual nsresult StartDocumentLoad(const char* aCommand,
+                                     nsIChannel* aChannel,
+                                     nsILoadGroup* aLoadGroup,
+                                     nsISupports* aContainer,
+                                     nsIStreamListener **aDocListener,
+                                     PRBool aReset = PR_TRUE,
+                                     nsIContentSink* aContentSink = nsnull);
 
-  NS_IMETHOD StartDocumentLoad(const char* aCommand,
-                               nsIChannel* aChannel,
-                               nsILoadGroup* aLoadGroup,
-                               nsISupports* aContainer,
-                               nsIStreamListener **aDocListener,
-                               PRBool aReset = PR_TRUE,
-                               nsIContentSink* aContentSink = nsnull);
-
-  NS_IMETHOD StopDocumentLoad();
-
-  /**
-   * Return the title of the document. May return null.
-   */
-  NS_IMETHOD GetDocumentTitle(nsAString& aTitle) const;
-
-  /**
-   * Return the URL for the document. May return null.
-   */
-  NS_IMETHOD GetDocumentURL(nsIURI** aURI) const;
+  virtual void StopDocumentLoad();
 
   /**
    * Return the principal responsible for this document.
    */
-  NS_IMETHOD GetPrincipal(nsIPrincipal **aPrincipal);
+  virtual nsIPrincipal* GetPrincipal();
 
   /**
-   * Update principal responsible for this document to the intersection
-   * of its previous value and aPrincipal, and return its new value.
+   * Set the principal responsible for this document.
    */
-  NS_IMETHOD AddPrincipal(nsIPrincipal *aPrincipal);
+  virtual void SetPrincipal(nsIPrincipal *aPrincipal);
 
   /**
    * Get the Content-Type of this document.
@@ -286,149 +229,124 @@ public:
   /**
    * Set the Content-Type of this document.
    */
-  NS_IMETHOD SetContentType(const nsAString& aContentType);
+  virtual void SetContentType(const nsAString& aContentType);
 
-  /**
-   * Return the content language of this document.
-   */
-  NS_IMETHOD GetContentLanguage(nsAString& aContentLanguage) const;
-
-  /**
-   * Return the LoadGroup for the document. May return null.
-   */
-  NS_IMETHOD GetDocumentLoadGroup(nsILoadGroup **aGroup) const;
-
-  /**
-   * Return the base URL for relative URLs in the document. May return null (or the document URL).
-   */
-  NS_IMETHOD GetBaseURL(nsIURI** aURL) const;
-  NS_IMETHOD SetBaseURL(nsIURI* aURL);
+  virtual nsresult SetBaseURI(nsIURI* aURI);
 
   /**
    * Get/Set the base target of a link in a document.
    */
-  NS_IMETHOD GetBaseTarget(nsAString &aBaseTarget);
-  NS_IMETHOD SetBaseTarget(const nsAString &aBaseTarget);
+  virtual void GetBaseTarget(nsAString &aBaseTarget) const;
+  virtual void SetBaseTarget(const nsAString &aBaseTarget);
 
   /**
    * Return a standard name for the document's character set. This will
    * trigger a startDocumentLoad if necessary to answer the question.
    */
-  NS_IMETHOD GetDocumentCharacterSet(nsACString& oCharsetID);
-  NS_IMETHOD SetDocumentCharacterSet(const nsACString& aCharSetID);
-
-  NS_IMETHOD GetDocumentCharacterSetSource(PRInt32* aCharsetSource);
-  NS_IMETHOD SetDocumentCharacterSetSource(PRInt32 aCharsetSource);
+  virtual void SetDocumentCharacterSet(const nsACString& aCharSetID);
 
   /**
    * Add an observer that gets notified whenever the charset changes.
    */
-  NS_IMETHOD AddCharSetObserver(nsIObserver* aObserver);
+  virtual nsresult AddCharSetObserver(nsIObserver* aObserver);
 
   /**
    * Remove a charset observer.
    */
-  NS_IMETHOD RemoveCharSetObserver(nsIObserver* aObserver);
-
-  /**
-   *  Check if the document contains bidi data.
-   *  If so, we have to apply the Unicode Bidi Algorithm.
-   */
-  NS_IMETHOD GetBidiEnabled(PRBool* aBidiEnabled) const;
-  /**
-   *  Indicate the document contains RTL characters.
-   */
-  NS_IMETHOD SetBidiEnabled(PRBool aBidiEnabled);
+  virtual void RemoveCharSetObserver(nsIObserver* aObserver);
 
   /**
    * Return the Line Breaker for the document
    */
-  NS_IMETHOD GetLineBreaker(nsILineBreaker** aResult)  ;
-  NS_IMETHOD SetLineBreaker(nsILineBreaker* aLineBreaker) ;
-  NS_IMETHOD GetWordBreaker(nsIWordBreaker** aResult)  ;
-  NS_IMETHOD SetWordBreaker(nsIWordBreaker* aWordBreaker) ;
+  virtual nsILineBreaker* GetLineBreaker();
+  virtual void SetLineBreaker(nsILineBreaker* aLineBreaker);
+  virtual nsIWordBreaker* GetWordBreaker();
+  virtual void SetWordBreaker(nsIWordBreaker* aWordBreaker);
 
   /**
    * Access HTTP header data (this may also get set from other sources, like
    * HTML META tags).
    */
-  NS_IMETHOD GetHeaderData(nsIAtom* aHeaderField,
-                           nsAString& aData) const;
-  NS_IMETHOD SetHeaderData(nsIAtom* aheaderField,
-                           const nsAString& aData);
+  virtual void GetHeaderData(nsIAtom* aHeaderField, nsAString& aData) const;
+  virtual void SetHeaderData(nsIAtom* aheaderField,
+                             const nsAString& aData);
 
   /**
    * Create a new presentation shell that will use aContext for
    * it's presentation context (presentation context's <b>must not</b> be
    * shared among multiple presentation shell's).
    */
-  NS_IMETHOD CreateShell(nsIPresContext* aContext,
-                         nsIViewManager* aViewManager,
-                         nsIStyleSet* aStyleSet,
-                         nsIPresShell** aInstancePtrResult);
+  virtual nsresult CreateShell(nsIPresContext* aContext,
+                               nsIViewManager* aViewManager,
+                               nsStyleSet* aStyleSet,
+                               nsIPresShell** aInstancePtrResult);
   virtual PRBool DeleteShell(nsIPresShell* aShell);
-  virtual PRInt32 GetNumberOfShells();
-  NS_IMETHOD GetShellAt(PRInt32 aIndex, nsIPresShell** aShell);
+  virtual PRUint32 GetNumberOfShells() const;
+  virtual nsIPresShell *GetShellAt(PRUint32 aIndex) const;
 
-  /**
-   * Return the parent document of this document. Will return null
-   * unless this document is within a compound document and has a parent.
-   */
-  NS_IMETHOD GetParentDocument(nsIDocument** aParent);
-  NS_IMETHOD SetParentDocument(nsIDocument* aParent);
+  virtual nsresult SetSubDocumentFor(nsIContent *aContent,
+                                     nsIDocument* aSubDoc);
+  virtual nsIDocument* GetSubDocumentFor(nsIContent *aContent) const;
+  virtual nsIContent* FindContentForSubDocument(nsIDocument *aDocument) const;
 
-  NS_IMETHOD SetSubDocumentFor(nsIContent *aContent, nsIDocument* aSubDoc);
-  NS_IMETHOD GetSubDocumentFor(nsIContent *aContent, nsIDocument** aSubDoc);
-  NS_IMETHOD FindContentForSubDocument(nsIDocument *aDocument,
-                                       nsIContent **aContent);
-
-  /**
-   * Return the root content object for this document.
-   */
-  NS_IMETHOD GetRootContent(nsIContent** aRoot);
-  NS_IMETHOD SetRootContent(nsIContent* aRoot);
+  virtual void SetRootContent(nsIContent* aRoot);
 
   /**
    * Get the direct children of the document - content in
    * the prolog, the root content and content in the epilog.
    */
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
-  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aIndex) const;
-  NS_IMETHOD GetChildCount(PRInt32& aCount);
+  virtual nsIContent *GetChildAt(PRUint32 aIndex) const;
+  virtual PRInt32 IndexOf(nsIContent* aPossibleChild) const;
+  virtual PRUint32 GetChildCount() const;
 
   /**
    * Get the style sheets owned by this document.
    * These are ordered, highest priority last
    */
-  NS_IMETHOD GetNumberOfStyleSheets(PRBool aIncludeSpecialSheets,
-                                    PRInt32* aCount);
-  NS_IMETHOD GetStyleSheetAt(PRInt32 aIndex, PRBool aIncludeSpecialSheets,
-                             nsIStyleSheet** aSheet);
-  NS_IMETHOD GetIndexOfStyleSheet(nsIStyleSheet* aSheet, PRInt32* aIndex);
+  virtual PRInt32 GetNumberOfStyleSheets(PRBool aIncludeSpecialSheets) const;
+  virtual nsIStyleSheet* GetStyleSheetAt(PRInt32 aIndex,
+                                         PRBool aIncludeSpecialSheets) const;
+  virtual PRInt32 GetIndexOfStyleSheet(nsIStyleSheet* aSheet) const;
   virtual void AddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags);
   virtual void RemoveStyleSheet(nsIStyleSheet* aSheet);
 
-  NS_IMETHOD UpdateStyleSheets(nsCOMArray<nsIStyleSheet>& aOldSheets,
-                               nsCOMArray<nsIStyleSheet>& aNewSheets);
+  virtual void UpdateStyleSheets(nsCOMArray<nsIStyleSheet>& aOldSheets,
+                                 nsCOMArray<nsIStyleSheet>& aNewSheets);
   virtual void AddStyleSheetToStyleSets(nsIStyleSheet* aSheet);
   virtual void RemoveStyleSheetFromStyleSets(nsIStyleSheet* aSheet);
 
-  NS_IMETHOD InsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex);
+  virtual void InsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex);
   virtual void SetStyleSheetApplicableState(nsIStyleSheet* aSheet,
                                             PRBool aApplicable);
 
+  /**
+   * Get this document's attribute stylesheet.  May return null if
+   * there isn't one.
+   */
+  nsIHTMLStyleSheet* GetAttributeStyleSheet() const {
+    return mAttrStyleSheet;
+  }
+
+  /**
+   * Get this document's inline style sheet.  May return null if there
+   * isn't one
+   */
+  nsIHTMLCSSStyleSheet* GetInlineStyleSheet() const {
+    return mStyleAttrStyleSheet;
+  }
+  
   /**
    * Set the object from which a document can get a script context.
    * This is the context within which all scripts (during document
    * creation and during event handling) will run.
    */
-  NS_IMETHOD GetScriptGlobalObject(nsIScriptGlobalObject** aGlobalObject);
-  NS_IMETHOD SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject);
+  virtual nsIScriptGlobalObject* GetScriptGlobalObject() const;
+  virtual void SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject);
 
   /**
    * Get the script loader for this document
    */
-  NS_IMETHOD GetScriptLoader(nsIScriptLoader** aScriptLoader);
+  virtual nsIScriptLoader* GetScriptLoader();
 
   /**
    * Add a new observer of document change notifications. Whenever
@@ -443,65 +361,63 @@ public:
    */
   virtual PRBool RemoveObserver(nsIDocumentObserver* aObserver);
 
-  // Observation hooks used by content nodes to propagate
-  // notifications to document observers.
-  NS_IMETHOD BeginUpdate();
-  NS_IMETHOD EndUpdate();
-  NS_IMETHOD BeginLoad();
-  NS_IMETHOD EndLoad();
-  NS_IMETHOD ContentChanged(nsIContent* aContent,
-                            nsISupports* aSubContent);
-  NS_IMETHOD ContentStatesChanged(nsIContent* aContent1,
-                                  nsIContent* aContent2,
-                                  PRInt32 aStateMask);
+  // Observation hooks used to propagate notifications to document
+  // observers.
+  virtual void BeginUpdate(nsUpdateType aUpdateType);
+  virtual void EndUpdate(nsUpdateType aUpdateType);
+  virtual void BeginLoad();
+  virtual void EndLoad();
+  virtual void CharacterDataChanged(nsIContent* aContent,
+                                    PRBool aAppend);
+  virtual void ContentStatesChanged(nsIContent* aContent1,
+                                    nsIContent* aContent2,
+                                    PRInt32 aStateMask);
 
-  NS_IMETHOD AttributeWillChange(nsIContent* aChild,
-                                 PRInt32 aNameSpaceID,
-                                 nsIAtom* aAttribute);
-  NS_IMETHOD AttributeChanged(nsIContent* aChild,
-                              PRInt32 aNameSpaceID,
-                              nsIAtom* aAttribute,
-                              PRInt32 aModType,
-                              nsChangeHint aHint);
-  NS_IMETHOD ContentAppended(nsIContent* aContainer,
-                             PRInt32 aNewIndexInContainer);
-  NS_IMETHOD ContentInserted(nsIContent* aContainer,
-                             nsIContent* aChild,
-                             PRInt32 aIndexInContainer);
-  NS_IMETHOD ContentReplaced(nsIContent* aContainer,
-                             nsIContent* aOldChild,
-                             nsIContent* aNewChild,
-                             PRInt32 aIndexInContainer);
-  NS_IMETHOD ContentRemoved(nsIContent* aContainer,
-                            nsIContent* aChild,
-                            PRInt32 aIndexInContainer);
+  virtual void AttributeWillChange(nsIContent* aChild,
+                                   PRInt32 aNameSpaceID,
+                                   nsIAtom* aAttribute);
+  virtual void AttributeChanged(nsIContent* aChild,
+                                PRInt32 aNameSpaceID,
+                                nsIAtom* aAttribute,
+                                PRInt32 aModType);
+  virtual void ContentAppended(nsIContent* aContainer,
+                               PRInt32 aNewIndexInContainer);
+  virtual void ContentInserted(nsIContent* aContainer,
+                               nsIContent* aChild,
+                               PRInt32 aIndexInContainer);
+  virtual void ContentReplaced(nsIContent* aContainer,
+                               nsIContent* aOldChild,
+                               nsIContent* aNewChild,
+                               PRInt32 aIndexInContainer);
+  virtual void ContentRemoved(nsIContent* aContainer,
+                              nsIContent* aChild,
+                              PRInt32 aIndexInContainer);
 
-  NS_IMETHOD StyleRuleChanged(nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aOldStyleRule,
-                              nsIStyleRule* aNewStyleRule);
-  NS_IMETHOD StyleRuleAdded(nsIStyleSheet* aStyleSheet,
-                            nsIStyleRule* aStyleRule);
-  NS_IMETHOD StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
+  virtual void StyleRuleChanged(nsIStyleSheet* aStyleSheet,
+                                nsIStyleRule* aOldStyleRule,
+                                nsIStyleRule* aNewStyleRule);
+  virtual void StyleRuleAdded(nsIStyleSheet* aStyleSheet,
                               nsIStyleRule* aStyleRule);
+  virtual void StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
+                                nsIStyleRule* aStyleRule);
 
-  NS_IMETHOD FlushPendingNotifications(PRBool aFlushReflows = PR_TRUE,
-                                       PRBool aUpdateViews = PR_FALSE);
-  NS_IMETHOD GetAndIncrementContentID(PRInt32* aID);
-  NS_IMETHOD GetBindingManager(nsIBindingManager** aResult);
-  NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager);
-  NS_IMETHOD AddReference(void *aKey, nsISupports *aReference);
-  NS_IMETHOD RemoveReference(void *aKey, nsISupports **aOldReference);
-  NS_IMETHOD SetContainer(nsISupports *aContainer);
-  NS_IMETHOD GetContainer(nsISupports **aContainer);
-  NS_IMETHOD GetScriptEventManager(nsIScriptEventManager **aResult);
-  NS_IMETHOD SetXMLDeclaration(const nsAString& aVersion,
+  virtual void FlushPendingNotifications(PRBool aFlushReflows = PR_TRUE,
+                                         PRBool aUpdateViews = PR_FALSE);
+  virtual void AddReference(void *aKey, nsISupports *aReference);
+  virtual already_AddRefed<nsISupports> RemoveReference(void *aKey);
+  virtual nsIScriptEventManager* GetScriptEventManager();
+  virtual void SetXMLDeclaration(const nsAString& aVersion,
                                const nsAString& aEncoding,
                                const nsAString& Standalone);
-  NS_IMETHOD GetXMLDeclaration(nsAString& aVersion,
-                               nsAString& aEncoding,
-                               nsAString& Standalone);
-  NS_IMETHOD_(PRBool) IsCaseSensitive();
-  NS_IMETHOD_(PRBool) IsScriptEnabled();
+  virtual void GetXMLDeclaration(nsAString& aVersion,
+                                 nsAString& aEncoding,
+                                 nsAString& Standalone);
+  virtual PRBool IsScriptEnabled();
+
+  virtual nsresult HandleDOMEvent(nsIPresContext* aPresContext,
+                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
+                                  PRUint32 aFlags,
+                                  nsEventStatus* aEventStatus);
 
   // nsIRadioGroupContainer
   NS_IMETHOD WalkRadioGroup(const nsAString& aName,
@@ -528,6 +444,9 @@ public:
   // nsIDOMDocument
   NS_DECL_NSIDOMDOCUMENT
 
+  // nsIDOM3Document
+  NS_DECL_NSIDOM3DOCUMENT
+
   // nsIDOMXMLDocument
   NS_DECL_NSIDOMXMLDOCUMENT
 
@@ -570,11 +489,9 @@ public:
   // nsIDOM3EventTarget
   NS_DECL_NSIDOM3EVENTTARGET
 
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
-                            nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
-                            nsEventStatus* aEventStatus);
-
-  NS_IMETHOD SetDocumentURL(nsIURI* aURI);
+  // nsIScriptObjectPrincipal
+  NS_IMETHOD GetPrincipalObsolete(nsIPrincipalObsolete **aPrincipal);
+  NS_IMETHOD GetPrincipal(nsIPrincipal **aPrincipal);
 
   virtual nsresult Init();
 
@@ -584,34 +501,37 @@ protected:
                                      PRUint32 aFlags);
   virtual void InternalInsertStyleSheetAt(nsIStyleSheet* aSheet,
                                           PRInt32 aIndex);
-  virtual already_AddRefed<nsIStyleSheet> InternalGetStyleSheetAt(PRInt32 aIndex);
-  virtual PRInt32 InternalGetNumberOfStyleSheets();
+  virtual nsIStyleSheet* InternalGetStyleSheetAt(PRInt32 aIndex) const;
+  virtual PRInt32 InternalGetNumberOfStyleSheets() const;
 
-  virtual void RetrieveRelevantHeaders(nsIChannel *aChannel);
+  void RetrieveRelevantHeaders(nsIChannel *aChannel);
 
+  static PRBool TryChannelCharset(nsIChannel *aChannel,
+                                  PRInt32& aCharsetSource,
+                                  nsACString& aCharset);
+  
   nsresult doCreateShell(nsIPresContext* aContext,
-                         nsIViewManager* aViewManager, nsIStyleSet* aStyleSet,
+                         nsIViewManager* aViewManager, nsStyleSet* aStyleSet,
                          nsCompatibility aCompatMode,
                          nsIPresShell** aInstancePtrResult);
 
+  nsresult ResetStylesheetsToURI(nsIURI* aURI);
+
+  nsresult CreateElement(nsINodeInfo *aNodeInfo, nsIDOMElement** aResult);
+
+  virtual PRInt32 GetDefaultNamespaceID() const
+  {
+    return kNameSpaceID_None;
+  };
+
   nsDocument();
   virtual ~nsDocument();
 
-  nsString mDocumentTitle;
+  nsCString mReferrer;
   nsCString mLastModified;
-  nsCOMPtr<nsIURI> mDocumentURL;
-  nsCOMPtr<nsIURI> mDocumentBaseURL;
   nsCOMPtr<nsIPrincipal> mPrincipal;
 
-  nsWeakPtr mDocumentLoadGroup;
-
-  nsWeakPtr mDocumentContainer;
-
-  nsCString mCharacterSet;
-  PRInt32 mCharacterSetSource;
-
   nsVoidArray mCharSetObservers;
-  nsIDocument* mParentDocument;
 
   PLDHashTable *mSubDocuments;
 
@@ -620,10 +540,6 @@ protected:
   // Array of owning references to all children
   nsCOMArray<nsIContent> mChildren;
 
-  // A weak reference to the only element in mChildren, or null if no
-  // such element exists.
-  nsIContent* mRootContent;
-
   nsCOMArray<nsIStyleSheet> mStyleSheets;
 
   // Basically always has at least 1 entry
@@ -638,21 +554,11 @@ protected:
 
   nsRefPtr<nsDocumentChildNodes> mChildNodes;
 
-  // A content ID counter used to give a monotonically increasing ID
-  // to the content objects in the document's content model
-  PRInt32 mNextContentID;
-
   nsHashtable mRadioGroups;
 
-  nsCOMPtr<nsIBindingManager> mBindingManager;
-  nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
-
   // True if the document is being destroyed.
   PRPackedBool mIsGoingAway;
 
-  // True if BIDI is enabled.
-  PRPackedBool mBidiEnabled;
-
   // True if the document is being destroyed.
   PRPackedBool mInDestructor;
 
@@ -663,13 +569,16 @@ protected:
   nsSupportsHashtable mContentWrapperHash;
 
   nsCOMPtr<nsICSSLoader> mCSSLoader;
-
-  nsXPIDLCString mContentLanguage;
-  nsCString mContentType;
+  nsCOMPtr<nsIHTMLStyleSheet> mAttrStyleSheet;
+  nsCOMPtr<nsIHTMLCSSStyleSheet> mStyleAttrStyleSheet;
 
   nsCOMPtr<nsIScriptEventManager> mScriptEventManager;
 
+  nsString mBaseTarget;
+
 private:
+  nsresult IsAllowedAsChild(PRUint16 aNodeType, nsIContent* aRefContent);
+
   // These are not implemented and not supported.
   nsDocument(const nsDocument& aOther);
   nsDocument& operator=(const nsDocument& aOther);
