@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -39,7 +39,6 @@
 #include "nsTextTransformer.h"
 #include "nsIContent.h"
 #include "nsIFrame.h"
-#include "nsIStyleContext.h"
 #include "nsITextContent.h"
 #include "nsStyleConsts.h"
 #include "nsILineBreaker.h"
@@ -49,15 +48,30 @@
 #include "nsUnicharUtils.h"
 #include "nsICaseConversion.h"
 #include "prenv.h"
-#include "nsIPref.h"
+#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #ifdef IBMBIDI
 #include "nsLayoutAtoms.h"
 #endif
 
 
-PRPackedBool nsTextTransformer::sWordSelectPrefInited = PR_FALSE;
-PRPackedBool nsTextTransformer::sWordSelectStopAtPunctuation = PR_FALSE;
+nsTextTransformer::WordSelectListener *nsTextTransformer::sWordSelectListener = nsnull;
+PRBool nsTextTransformer::sWordSelectStopAtPunctuation = PR_FALSE;
+static const char kWordSelectPref[] = "layout.word_select.stop_at_punctuation";
 
+NS_IMPL_ISUPPORTS1(nsTextTransformer::WordSelectListener, nsIObserver)
+
+NS_IMETHODIMP
+nsTextTransformer::WordSelectListener::Observe(nsISupports *aSubject,
+                                                 const char *aTopic,
+                                                 const PRUnichar *aData)
+{
+  NS_ASSERTION(!nsCRT::strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID),
+               "wrong topic");
+  nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(aSubject);
+  return prefBranch->GetBoolPref(kWordSelectPref, &sWordSelectStopAtPunctuation);
+}
 
 nsAutoTextBuffer::nsAutoTextBuffer()
   : mBuffer(mAutoBuffer),
@@ -110,20 +124,21 @@ static nsICaseConversion* gCaseConv =  nsnull;
 nsresult
 nsTextTransformer::Initialize()
 {
-  nsresult res = NS_OK;
-  
   // read in our global word selection prefs
-  if ( !sWordSelectPrefInited ) {
-    nsCOMPtr<nsIPref> prefService ( do_GetService(NS_PREF_CONTRACTID) );
-    if ( prefService ) {
-      PRBool temp = PR_FALSE;
-      prefService->GetBoolPref("layout.word_select.stop_at_punctuation", &temp);
-      sWordSelectStopAtPunctuation = temp;
+  if ( !sWordSelectListener ) {
+    nsCOMPtr<nsIPrefBranchInternal> prefBranch =
+        do_GetService( NS_PREFSERVICE_CONTRACTID );
+    if ( prefBranch ) {
+      prefBranch->GetBoolPref(kWordSelectPref, &sWordSelectStopAtPunctuation);
+      sWordSelectListener = new WordSelectListener();
+      if (sWordSelectListener) {
+        NS_ADDREF(sWordSelectListener);
+        prefBranch->AddObserver(kWordSelectPref, sWordSelectListener, PR_FALSE);
+      }
     }
-    sWordSelectPrefInited = PR_TRUE;
   }
-  
-  return res;
+
+  return NS_OK;
 }
 static nsresult EnsureCaseConv()
 {
@@ -140,6 +155,7 @@ static nsresult EnsureCaseConv()
 void
 nsTextTransformer::Shutdown()
 {
+  NS_IF_RELEASE(sWordSelectListener);
   if (gCaseConv) {
     nsServiceManager::ReleaseService(kUnicharUtilCID, gCaseConv);
     gCaseConv = nsnull;
@@ -204,23 +220,40 @@ nsresult
 nsTextTransformer::Init(nsIFrame* aFrame,
                         nsIContent* aContent,
                         PRInt32 aStartingOffset,
+                        PRBool aForceArabicShaping,
                         PRBool aLeaveAsAscii)
 {
-#ifdef IBMBIDI
+  /*
+   * If the document has Bidi content, check whether we need to do
+   * Arabic shaping.
+   *
+   *  Does the frame contains Arabic characters
+   *   (mCharType == eCharType_RightToLeftArabic)?
+   *  Are we rendering character by character (aForceArabicShaping ==
+   *   PR_TRUE)? If so, we always do our own Arabic shaping, even if
+   *   the platform has native shaping support. Otherwise, we only do
+   *   shaping if the platform has no shaping support.
+   *
+   *  We do numeric shaping in all Bidi documents.
+   */
   PRBool bidiEnabled;
 
   mPresContext->GetBidiEnabled(&bidiEnabled);
   if (bidiEnabled) {
-    PRBool isBidiSystem;
     aFrame->GetBidiProperty(mPresContext, nsLayoutAtoms::charType,
                             (void**)&mCharType, sizeof(mCharType));
-    mPresContext->GetIsBidiSystem(isBidiSystem);
-    if (mCharType == eCharType_RightToLeftArabic && !isBidiSystem) {
-      SetNeedsArabicShaping(PR_TRUE);
+    if (mCharType == eCharType_RightToLeftArabic) {
+      if (aForceArabicShaping) {
+        SetNeedsArabicShaping(PR_TRUE);
+      }
+      else {
+        if (!mPresContext->IsBidiSystem()) {
+          SetNeedsArabicShaping(PR_TRUE);
+        }
+      }
     }
     SetNeedsNumericShaping(PR_TRUE);
   }
-#endif
 
   // Get the contents text content
   nsresult rv;
@@ -240,8 +273,7 @@ nsTextTransformer::Init(nsIFrame* aFrame,
     mOffset = aStartingOffset;
 
     // Get the frames text style information
-    const nsStyleText* styleText;
-    aFrame->GetStyleData(eStyleStruct_Text, (const nsStyleStruct*&) styleText);
+    const nsStyleText* styleText = aFrame->GetStyleText();
     if (NS_STYLE_WHITESPACE_PRE == styleText->mWhiteSpace) {
       mMode = ePreformatted;
     }
@@ -393,7 +425,8 @@ nsTextTransformer::ScanNormalAsciiText_F(PRInt32* aWordLen,
 
 PRInt32
 nsTextTransformer::ScanNormalAsciiText_F_ForWordBreak(PRInt32* aWordLen,
-                                         PRBool*  aWasTransformed)
+                                         PRBool*  aWasTransformed,
+                                         PRBool aIsKeyboardSelect)
 {
   const nsTextFragment* frag = mFrag;
   PRInt32 fragLen = frag->GetLength();
@@ -411,7 +444,16 @@ nsTextTransformer::ScanNormalAsciiText_F_ForWordBreak(PRInt32* aWordLen,
   } else {
     bp2 += mBufferPos;
   }
+  PRBool readingAlphaNumeric = PR_TRUE; //only used in sWordSelectStopAtPunctuation
 
+  // We must know if we are starting in alpha numerics.
+  // Treat high bit chars as alphanumeric, otherwise we get stuck on accented letters
+  // We can't trust isalnum() results for isalnum()
+  // Therefore we don't stop at non-ascii (high bit) punctuation,
+  // which is just fine. The punctuation we care about is low bit.
+  if (sWordSelectStopAtPunctuation && offset < fragLen)
+    readingAlphaNumeric = isalnum((unsigned char)*cp) || !IS_ASCII_CHAR(*cp);
+  
   for (; offset < fragLen && !breakAfterThis; offset++) {
     unsigned char ch = *cp++;
     if (CH_NBSP == ch) {
@@ -425,8 +467,17 @@ nsTextTransformer::ScanNormalAsciiText_F_ForWordBreak(PRInt32* aWordLen,
     else if (XP_IS_SPACE(ch)) {
       break;
     }
-    else if (sWordSelectStopAtPunctuation && !isalnum(ch)) {
-      // on some platforms, punctuation breaks words too.
+    else if (sWordSelectStopAtPunctuation && 
+             readingAlphaNumeric && !isalnum(ch) && IS_ASCII_CHAR(ch)) {
+      if (!aIsKeyboardSelect)
+        break;
+      // For keyboard move-by-word, need to pass by at least
+      // one alphanumeric char before stopping at punct
+      readingAlphaNumeric = PR_FALSE;
+    }
+    else if (sWordSelectStopAtPunctuation && 
+            !readingAlphaNumeric && (isalnum(ch) || !IS_ASCII_CHAR(ch))) {
+      // On some platforms, punctuation breaks for word selection
       break;
     }
     else if (IS_DISCARDED(ch)) {
@@ -539,7 +590,7 @@ nsTextTransformer::ScanNormalUnicodeText_F(PRBool   aForLineBreak,
         mLineBreaker->Next(cp0, fragLen, offset, &next, &tryNextFrag);
       }
       else {
-        mWordBreaker->Next(cp0, fragLen, offset, &next, &tryNextFrag);
+        mWordBreaker->NextWord(cp0, fragLen, offset, &next, &tryNextFrag);
       }
       numChars = (PRInt32) (next - (PRUint32) offset) + 1;
 
@@ -811,7 +862,8 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
                                PRBool* aIsWhiteSpaceResult,
                                PRBool* aWasTransformed,
                                PRBool aResetTransformBuf,
-                               PRBool aForLineBreak)
+                               PRBool aForLineBreak,
+                               PRBool aIsKeyboardSelect)
 {
   const nsTextFragment* frag = mFrag;
   PRInt32 fragLen = frag->GetLength();
@@ -904,7 +956,9 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
         }
         else {
           if (!aForLineBreak)
-            offset = ScanNormalAsciiText_F_ForWordBreak(&wordLen, aWasTransformed);
+            offset = ScanNormalAsciiText_F_ForWordBreak(&wordLen, 
+                                                        aWasTransformed, 
+                                                        aIsKeyboardSelect);
           else
             offset = ScanNormalAsciiText_F(&wordLen, aWasTransformed);
         }
@@ -945,7 +999,8 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
         }
         else {
           if (!aForLineBreak)
-            offset = ScanNormalAsciiText_F_ForWordBreak(&wordLen, aWasTransformed);
+            offset = ScanNormalAsciiText_F_ForWordBreak(&wordLen, aWasTransformed, 
+                                                        aIsKeyboardSelect);
           else
             offset = ScanNormalAsciiText_F(&wordLen, aWasTransformed);
         }
@@ -1027,14 +1082,12 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
             eLanguageSpecificTransformType_None) {
           LanguageSpecificTransform(result, wordLen, aWasTransformed);
         }
-#ifdef IBMBIDI
         if (NeedsArabicShaping()) {
           DoArabicShaping(result, wordLen, aWasTransformed);
         }
         if (NeedsNumericShaping()) {
           DoNumericShaping(result, wordLen, aWasTransformed);
         }
-#endif
       }
     }
 
@@ -1053,9 +1106,12 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
   if ((mTextTransform != NS_STYLE_TEXT_TRANSFORM_NONE) ||
       (*aWordLenResult != *aContentLenResult)) {
     *aWasTransformed = PR_TRUE;
+    mBufferPos = prevBufferPos + *aWordLenResult;
   }
 
   mOffset = offset;
+
+  NS_ASSERTION(mBufferPos == prevBufferPos + *aWordLenResult, "internal error");
   return result;
 }
 
@@ -1085,13 +1141,20 @@ nsTextTransformer::ScanNormalWhiteSpace_B()
 
 // wordlen==*aWordLen, contentlen=newOffset-currentOffset, isWhitespace=f
 PRInt32
-nsTextTransformer::ScanNormalAsciiText_B(PRInt32* aWordLen)
+nsTextTransformer::ScanNormalAsciiText_B(PRInt32* aWordLen, PRBool aIsKeyboardSelect)
 {
   const nsTextFragment* frag = mFrag;
   PRInt32 offset = mOffset;
   PRUnichar* bp = mTransformBuf.GetBufferEnd();
   PRUnichar* startbp = mTransformBuf.GetBuffer();
 
+  PRUnichar ch = frag->CharAt(offset - 1);
+  // Treat high bit chars as alphanumeric, otherwise we get stuck on accented letters
+  // We can't trust isalnum() results for isalnum()
+  // Therefore we don't stop at non-ascii (high bit) punctuation,
+  // which is just fine. The punctuation we care about is low bit.
+  PRBool readingAlphaNumeric = isalnum(ch) || !IS_ASCII_CHAR(ch);
+
   while (--offset >= 0) {
     PRUnichar ch = frag->CharAt(offset);
     if (CH_NBSP == ch) {
@@ -1103,6 +1166,18 @@ nsTextTransformer::ScanNormalAsciiText_B(PRInt32* aWordLen)
     else if (IS_DISCARDED(ch)) {
       continue;
     } 
+    else if (sWordSelectStopAtPunctuation && readingAlphaNumeric && 
+             !isalnum(ch) && IS_ASCII_CHAR(ch)) {
+      // Break on ascii punctuation
+      break;
+    }
+    else if (sWordSelectStopAtPunctuation && !readingAlphaNumeric &&
+             (isalnum(ch) || !IS_ASCII_CHAR(ch))) {
+      if (!aIsKeyboardSelect)
+        break;
+      readingAlphaNumeric = PR_TRUE;
+    }
+    
     if (ch > MAX_UNIBYTE) SetHasMultibyte(PR_TRUE);
     if (bp == startbp) {
       PRInt32 oldLength = mTransformBuf.mBufferLen;
@@ -1171,7 +1246,7 @@ nsTextTransformer::ScanNormalUnicodeText_B(PRBool aForLineBreak,
         mLineBreaker->Prev(cp0, offset, offset, &prev, &tryPrevFrag);
       }
       else {
-        mWordBreaker->Prev(cp0, offset, offset, &prev, &tryPrevFrag);
+        mWordBreaker->PrevWord(cp0, offset, offset, &prev, &tryPrevFrag);
       }
       numChars = (PRInt32) ((PRUint32) offset - prev) + 1;
 
@@ -1291,7 +1366,8 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
                                PRInt32* aWordLenResult,
                                PRInt32* aContentLenResult,
                                PRBool* aIsWhiteSpaceResult,
-                               PRBool aForLineBreak)
+                               PRBool aForLineBreak,
+                               PRBool aIsKeyboardSelect)
 {
   const nsTextFragment* frag = mFrag;
   PRInt32 offset = mOffset;
@@ -1337,7 +1413,7 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
           offset = ScanNormalUnicodeText_B(aForLineBreak, &wordLen);
         }
         else {
-          offset = ScanNormalAsciiText_B(&wordLen);
+          offset = ScanNormalAsciiText_B(&wordLen, aIsKeyboardSelect);
         }
         break;
 
@@ -1372,7 +1448,7 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
           offset = ScanNormalUnicodeText_B(aForLineBreak, &wordLen);
         }
         else {
-          offset = ScanNormalAsciiText_B(&wordLen);
+          offset = ScanNormalAsciiText_B(&wordLen, aIsKeyboardSelect);
         }
         break;
     }
@@ -1410,7 +1486,6 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
   return result;
 }
 
-#ifdef IBMBIDI
 void
 nsTextTransformer::DoArabicShaping(PRUnichar* aText, 
                                    PRInt32& aTextLength, 
@@ -1420,28 +1495,18 @@ nsTextTransformer::DoArabicShaping(PRUnichar* aText,
     return;
   
   PRInt32 newLen;
-  PRBool isVisual;
-  mPresContext->IsVisualMode(isVisual);
+  PRBool isVisual = mPresContext->IsVisualMode();
 
   nsAutoString buf;
   buf.SetLength(aTextLength);
-  PRUnichar* buffer = (PRUnichar*)buf.get();
+  PRUnichar* buffer = buf.BeginWriting();
   
   ArabicShaping(aText, buf.Length(), buffer, (PRUint32 *)&newLen, !isVisual, !isVisual);
 
-  PRUnichar *source = buffer;
-  PRUnichar *target = aText;
-  for (PRInt32 i = 0; i < newLen; i++) {
-    if (*source == CH_ZWNJ || *source == CH_ZWJ) {
-      source++;
-    }
-    else {
-      *target++ = *source++;
-    }
-  }
-
-  aTextLength = target - aText;
+  aTextLength = newLen;
   *aWasTransformed = PR_TRUE;
+
+  StripZeroWidthJoinControls(buffer, aText, aTextLength, aWasTransformed);
 }
 
 void
@@ -1491,11 +1556,34 @@ nsTextTransformer::DoNumericShaping(PRUnichar* aText,
         HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_ARABIC);
       break;
 
+    case IBMBIDI_NUMERAL_NOMINAL:
     default:
       break;
   }
 }
-#endif
+
+void
+nsTextTransformer::StripZeroWidthJoinControls(PRUnichar* aSource,
+                                              PRUnichar* aTarget,
+                                              PRInt32& aTextLength, 
+                                              PRBool* aWasTransformed)
+{
+  PRUnichar *src, *dest;
+  PRInt32 stripped = 0;
+
+  src = aSource;
+  dest = aTarget;
+
+  for (PRInt32 i = 0; i < aTextLength; ++i) {
+    while (*src == CH_ZWNJ || *src == CH_ZWJ) {
+      ++stripped;
+      ++src;
+      *aWasTransformed = PR_TRUE;
+    }
+    *dest++ = *src++;
+  }
+  aTextLength -= stripped;
+}
 
 //----------------------------------------------------------------------
 // Self test logic for this class. This will (hopefully) make sure
@@ -1651,7 +1739,7 @@ nsTextTransformer::SelfTest(nsILineBreaker* aLineBreaker,
       if (gNoisy) {
         nsAutoString uc2(st->text);
         printf("%s forwards test: '", isAsciiTest ? "ascii" : "unicode");
-        fputs(NS_LossyConvertUCS2toASCII(uc2).get(), stdout);
+        fputs(NS_ConvertUCS2toUTF8(uc2).get(), stdout);
         printf("'\n");
       }
       tx.Init2(&frag, 0, preModeValue[preMode], NS_STYLE_TEXT_TRANSFORM_NONE);
@@ -1666,7 +1754,7 @@ nsTextTransformer::SelfTest(nsILineBreaker* aLineBreaker,
         if (gNoisy) {
           nsAutoString tmp(bp, wordLen);
           printf("  '");
-          fputs(NS_LossyConvertUCS2toASCII(tmp).get(), stdout);
+          fputs(NS_ConvertUCS2toUTF8(tmp).get(), stdout);
           printf("': ws=%s wordLen=%d (%d) contentLen=%d (offset=%d)\n",
                  ws ? "yes" : "no",
                  wordLen, *expectedResults, contentLen, tx.mOffset);
@@ -1690,7 +1778,7 @@ nsTextTransformer::SelfTest(nsILineBreaker* aLineBreaker,
       if (gNoisy) {
         nsAutoString uc2(st->text);
         printf("%s backwards test: '", isAsciiTest ? "ascii" : "unicode");
-        fputs(NS_LossyConvertUCS2toASCII(uc2).get(), stdout);
+        fputs(NS_ConvertUCS2toUTF8(uc2).get(), stdout);
         printf("'\n");
       }
       tx.Init2(&frag, frag.GetLength(), NS_STYLE_WHITESPACE_NORMAL,
@@ -1704,7 +1792,7 @@ nsTextTransformer::SelfTest(nsILineBreaker* aLineBreaker,
         if (gNoisy) {
           nsAutoString tmp(bp, wordLen);
           printf("  '");
-          fputs(NS_LossyConvertUCS2toASCII(tmp).get(), stdout);
+          fputs(NS_ConvertUCS2toUTF8(tmp).get(), stdout);
           printf("': ws=%s wordLen=%d contentLen=%d (offset=%d)\n",
                  ws ? "yes" : "no",
                  wordLen, contentLen, tx.mOffset);
@@ -1726,6 +1814,10 @@ nsTextTransformer::SelfTest(nsILineBreaker* aLineBreaker,
       if (error) {
         fprintf(stderr, "nsTextTransformer: self test %d failed\n", testNum);
       }
+      else if (gNoisy) {
+        fprintf(stdout, "nsTextTransformer: self test %d succeeded\n", testNum);
+      }
+
       testNum++;
     }
   }
