@@ -888,9 +888,8 @@ class nsDirEnumerator : public nsISimpleEnumerator
             return NS_OK;
         }
 
-        virtual ~nsDirEnumerator() 
-        {
-        }
+    private:
+        ~nsDirEnumerator() {}
 
     protected:
         nsCOMPtr<nsILocalFileMac>   mNext;
@@ -966,10 +965,6 @@ nsLocalFile& nsLocalFile::operator=(const nsLocalFile& rhs)
     return *this;
 }
 
-nsLocalFile::~nsLocalFile()
-{
-}
-
 #pragma mark -
 #pragma mark [nsISupports interface implementation]
 
@@ -1084,15 +1079,11 @@ nsLocalFile::ResolveAndStat()
 NS_IMETHODIMP  
 nsLocalFile::Clone(nsIFile **file)
 {
-    NS_ENSURE_ARG(file);
-    *file = nsnull;
-    
     // Just copy-construct ourselves
-    nsCOMPtr<nsILocalFile> localFile = new nsLocalFile(*this);
-    if (localFile == NULL)
+    *file = new nsLocalFile(*this);
+    if (!*file)
       return NS_ERROR_OUT_OF_MEMORY;
 
-    *file = localFile;
     NS_ADDREF(*file);
     
     return NS_OK;
